==> theremin_1.4.0-2.tar.gz: theremin-1.4.0/server/src/Therver.cpp <==
// (c) Ivan Gagis
// e-mail: igagis@gmail.com
// Version: 1

// Description:
//	Theremin server

#include <ting/Ref.hpp>
#include <ting/Array.hpp>
#include <ting/Thread.hpp>
#include <ting/types.hpp>
#include <ting/utils.hpp>

#include "Therver.hpp"
#include "Cast.hpp"
#include "Client.hpp"
#include "DatabaseThread.hpp"


using namespace ting;



//override
ting::Ref<cliser::Client> Therver::CreateClientObject(){
	return ting::Ref<cliser::Client>(new ::Client());
}



//override
void Therver::OnClientConnected(ting::Ref<cliser::Client>& client){
	ASSERT(client.IsValid())
	ASSERT(client.DynamicCast<Client>().IsValid())
	Ref<Client> c = client.StaticCast<Client>();

	TRACE(<< "!!!! CLIENT CONNECTED !!!!" << std::endl)
}



//override
void Therver::OnClientDisconnected(ting::Ref<cliser::Client>& client){
	ASSERT(client.IsValid())
	ASSERT(client.DynamicCast<Client>().IsValid())
	Ref<Client> c = client.StaticCast<Client>();
	
	TRACE(<< "!!!! CLIENT DISCONNECTED !!!!" << std::endl)

	this->PushMessage(Ptr<Message>(
			new ClientDisconnectedMessage(this, c)
		));
}



//override
void Therver::OnDataReceivedFromClient(ting::Ref<cliser::Client>& client, ting::Array<ting::byte> d){
	ASSERT(client.IsValid())
	ASSERT(client.DynamicCast<Client>().IsValid())
	Ref<Client> c = client.StaticCast<Client>();
	
	ASSERT(d.Size() > 0)
	
	TRACE(<< "!!!! DATA RECEIVED !!!! " << d.SizeInBytes() << " bytes, d[0] = " << u32(d[0])  << std::endl)

	this->PushMessage(Ptr<Message>(
			new NetworkDataFromClientReceivedMessage(this, c, d)
		));
}



//override
void ClientDisconnectedMessage::Handle(){
	this->c->SetState(Client::DISCONNECTED);

	//remove the client from cast it listening to, if it does
	if(this->c->isListening){
		(*this->c->listeningCast).RemoveListener(this->c);
		this->c->isListening = false;
	}

	//if client has a playing cast then notify listeners that the cast has finished
	//and remove the cast
	if(this->c->isPlaying){
		this->t->RemoveCast(this->c->playingCast);
	}
}



//override
void NetworkDataFromClientReceivedMessage::Handle(){
	switch(this->data[0]){
		case 0x00: //log in
			this->thr->HandleLoginFromClient(this->client, this->data);
			break;
		case 0x01: //listen next player
			this->thr->HandleListenNextCommandFromClient(this->client);
			break;
		case 0x02:
			this->thr->HandleStopListeningCommandFromClient(this->client);
			break;
		case 0x03: //playing
			this->thr->HandlePlayingDataFromClient(this->client, this->data);
			break;
		case 0x04: //register request
			this->thr->HandleRegisterFromClient(this->client, this->data);
			break;
		default:
			ASSERT_INFO(false, "unknown packet ID = " << u32(this->data[0]))
			break;
	}
}



void Therver::HandleRegisterFromClient(ting::Ref< ::Client>& c, ting::Array<ting::byte> d){
	ASSERT(c)
	ASSERT(d.Size() > 0)
	ASSERT(d[0] == 0x04) //4 = register command

	ting::uint i = 1;

	if(d.Size() < i + 2){
		ASSERT_INFO(false, "bad formed packet")
		return;
	}
	ting::uint unLen = ting::FromNetworkFormat16(&d[i]);
	i += 2;

	if(d.Size() < i + unLen){
		ASSERT_INFO(false, "bad formed packet")
		return;
	}
	std::string un(reinterpret_cast<char*>(&d[i]), unLen);
	i += unLen;

	if(d.Size() < i + 2){
		ASSERT_INFO(false, "bad formed packet")
		return;
	}
	ting::uint pwLen = ting::FromNetworkFormat16(&d[i]);
	i += 2;

	if(d.Size() < i + pwLen){
		ASSERT_INFO(false, "bad formed packet")
		return;
	}
	std::string pw(reinterpret_cast<char*>(&d[i]), pwLen);

	ASSERT(i + pwLen == d.Size())

	//send request to DatabaseThread
	this->dbThread.PushMessage(Ptr<Message>(
			new RegisterNewUserMessage(
					&this->dbThread,
					c,
					un,
					pw
				)
		));
}



void Therver::HandleLoginFromClient(ting::Ref< ::Client>& c, ting::Array<ting::byte> d){
	ASSERT(c)
	ASSERT(d[0] == 0)// 0 = login command

	//vars to be filled
	std::string un;
	std::string pw;

	ting::uint i = 1;

	//parse username
	{
		if(d.SizeInBytes() - i < 2){
			TRACE(<< "Therver::HandleLoginFromClient(): bad formed login packet !!! IGNORING !!!" << std::endl)
			return;
		}

//		TRACE(<< "unLen = " << u32(d[i]) << " " << u32(d[i + 1]) << std::endl)

		ting::uint usernameLength = ting::FromNetworkFormat16(&d[i]);
		i += 2;

		if(d.SizeInBytes() - i < usernameLength){
			TRACE(<< "Therver::HandleLoginFromClient(): bad formed login packet !!! IGNORING !!!" << std::endl)
			return;
		}

		un = std::string(reinterpret_cast<char*>(&d[i]), usernameLength);
		i += usernameLength;
	}

	//parse password
	{
		if(d.SizeInBytes() - i < 2){
			TRACE(<< "Therver::HandleLoginFromClient(): bad formed login packet !!! IGNORING !!!" << std::endl)
			return;
		}

//		TRACE(<< "pwLen = " << u32(d[i]) << " " << u32(d[i + 1]) << std::endl)

		ting::uint passwordLength = ting::FromNetworkFormat16(&d[i]);
		i += 2;

		if(d.SizeInBytes() - i < passwordLength){
			TRACE(<< "Therver::HandleLoginFromClient(): bad formed login packet !!! IGNORING !!!" << std::endl)
			return;
		}

		pw = std::string(reinterpret_cast<char*>(&d[i]), passwordLength);
		i += passwordLength;
	}

	ASSERT(i == d.SizeInBytes())

	//empty login and password are not allowed
	if(un.size() == 0 || pw.size() == 0){
		c->SendLoginResult(false);
		ASSERT(false)
		return;
	}

	c->username = un;
	c->password = pw;

	c->SetState(Client::LOGGING_IN);

	//send query request to DB thread
	this->dbThread.PushMessage(Ptr<Message>(
			new QueryUserPasswordMessage(
					&this->dbThread,
					c,
					c->username
				)
		));
}



void Therver::HandlePlayingDataFromClient(ting::Ref< ::Client>& c, ting::Array<ting::byte> d){
	//if client is not logged in then ignore that
	if(c->IsNotLoggedIn()){
		ASSERT_INFO(false, "client is not logged in")
		return;
	}

	//if the client does not play and does not have its own cast
	//then create one, because we received playing data from player
	//which means it started playing
	if(!c->isPlaying){
		//create new cast
		this->casts.push_back(Cast(c));

		ASSERT(this->casts.size() > 0)
		T_CastsIter castIter = (--this->casts.end());

		c->isPlaying = true;
		c->playingCast = castIter;
	}

	ASSERT(c->isPlaying)

	c->playingCast->PushNewMusicDataPacket(d);
}



void Therver::HandleStopListeningCommandFromClient(ting::Ref<Client>& c){
	if(!c->isListening){
		//already not listening
		return;
	}

	ASSERT(c->listeningCast != this->casts.end())

	c->isListening = false;
	(*c->listeningCast).RemoveListener(c);
}



void Therver::HandleListenNextCommandFromClient(ting::Ref< ::Client>& c){
	//if client is not logged in then ignore that
	if(c->IsNotLoggedIn()){
		ASSERT_INFO(false, "client is not logged in")
		return;
	}

	if(!c->isListening){
		c->listeningCast = this->casts.begin();
		c->isListening = true;
	}else{
		ASSERT(c->listeningCast != this->casts.end())
		(*c->listeningCast).RemoveListener(c);
		++c->listeningCast;
	}

	//if we reached end try to jump to the beginning
	if(c->listeningCast == this->casts.end()){
		if(this->casts.size() > 0){
			c->listeningCast = this->casts.begin();
			ASSERT(c->listeningCast != this->casts.end())
		}else{
			//there is no one to listen to
			c->isListening = false;
			c->listeningCast = this->casts.end();
			c->SendStopListening();//send 'stop listening' to client
			return;
		}
	}

	(*c->listeningCast).AddListenerAndSendDataToClient(c);
}



T_CastsIter Therver::RemoveCast(T_CastsIter castIter){
	ASSERT(castIter != this->casts.end())
	Cast& cast = *castIter;

	T_CastsIter nextCastIter = castIter;
	++nextCastIter;

	if(nextCastIter == this->casts.end()){
		if(this->casts.size() > 1){
			nextCastIter = this->casts.begin();
		}
	}

	ASSERT(nextCastIter != castIter)

	if(nextCastIter != this->casts.end()){
		Cast& nextCast = *nextCastIter;

		while(Ref<Client> c = cast.PopListener()){
			ASSERT(c->isListening)
			ASSERT(c->listeningCast == castIter)

			c->listeningCast = nextCastIter;
			nextCast.AddListenerAndSendDataToClient(c);
		}//~while
	}else{//there is no one to listen to
		while(Ref<Client> c = cast.PopListener()){
			ASSERT(c->isListening)
			ASSERT(c->listeningCast == castIter)

			c->isListening = false;
			c->listeningCast = this->casts.end();
			c->SendStopListening();//send 'stop listening' to client
		}//~while
	}

	if(Ref<Client> plr = cast.GetPlayer()){
		plr->isPlaying = false;
		plr->playingCast = this->casts.end();
	}

	return this->casts.erase(castIter);
}



//override
void UserPasswordQueriedMessage::Handle(){
	ASSERT(this->c)
	
	if(this->res != SUCCESS){
		this->c->SetState(Client::CONNECTED);
		this->c->SendLoginResult(false);//send login failure
		return;
	}

	//compare passwords
	if(this->c->password.compare(this->pw) != 0){
		this->c->SetState(Client::CONNECTED);
		this->c->SendLoginResult(false);//send login failure
		return;
	}

	this->c->SetState(Client::LOGGED_IN);
	this->c->SendLoginResult(true);//send login success
}



void Therver::PushRegisterResultMessage(ERegisterNewUserResult result, ting::Ref<Client> c){
	this->PushMessage(Ptr<Message>(
			new NewUserRegisteredMessage(this, c, result)
		));
}



//override
void NewUserRegisteredMessage::Handle(){
	Array<byte> p(2);

	p[0] = 0x04; //register response

	switch(this->res){
		case Therver::REGISTERRES_SUCCESS:
			p[1] = 0;
			break;
		case Therver::REGISTERRES_USERNAME_ALREADY_TAKEN:
			p[1] = 1;
			break;
		default:
		case Therver::REGISTERRES_SOME_ERROR:
			p[1] = 3;
			break;
	}

	//send response to client
	this->c->SendNetworkData_ts(p);
}



//override
u32 Therver::KillIdleCastsTimer::OnExpire(){
	this->t->PushMessage(Ptr<Message>(
			new KillIdleCastsMessage(this->t)
		));
	return 0;//do not reschedule timer
}



void Therver::KillIdleCasts(){
	TRACE(<< "Therver::KillIdleCasts(): invoked" << std::endl)
	u32 curTicks = ting::GetTicks();

	for(T_CastsIter i = this->casts.begin(); i != this->casts.end(); ){
		if(curTicks - (*i).GetLastChunkArrivalTimestamp() >= Therver::DIdleCastDuration()){
			i = this->RemoveCast(i);
			TRACE(<< "cast killed" << std::endl)
		}else{
			++i;
		}
	}

	this->killIdleCastsTimer.StartTimer();//schedule timer again
}

==> theremin_1.4.0-3.tar.gz: theremin-1.4.0/server/src/Therver.cpp <==
// (c) Ivan Gagis
// e-mail: igagis@gmail.com
// Version: 1

// Description:
//	Theremin server

#include <ting/Ref.hpp>
#include <ting/Array.hpp>
#include <ting/Thread.hpp>
#include <ting/types.hpp>
#include <ting/utils.hpp>

#include "Therver.hpp"
#include "Cast.hpp"
#include "Client.hpp"
#include "DatabaseThread.hpp"


using namespace ting;



//override
ting::Ref<cliser::Client> Therver::CreateClientObject(){
	return ting::Ref<cliser::Client>(new ::Client());
}



//override
void Therver::OnClientConnected(ting::Ref<cliser::Client>& client){
	ASSERT(client.IsValid())
	ASSERT(client.DynamicCast<Client>().IsValid())
	Ref<Client> c = client.StaticCast<Client>();

	TRACE(<< "!!!! CLIENT CONNECTED !!!!" << std::endl)
}



//override
void Therver::OnClientDisconnected(ting::Ref<cliser::Client>& client){
	ASSERT(client.IsValid())
	ASSERT(client.DynamicCast<Client>().IsValid())
	Ref<Client> c = client.StaticCast<Client>();
	
	TRACE(<< "!!!! CLIENT DISCONNECTED !!!!" << std::endl)

	this->PushMessage(Ptr<Message>(
			new ClientDisconnectedMessage(this, c)
		));
}



//override
void Therver::OnDataReceivedFromClient(ting::Ref<cliser::Client>& client, ting::Array<ting::byte> d){
	ASSERT(client.IsValid())
	ASSERT(client.DynamicCast<Client>().IsValid())
	Ref<Client> c = client.StaticCast<Client>();
	
	ASSERT(d.Size() > 0)
	
	TRACE(<< "!!!! DATA RECEIVED !!!! " << d.SizeInBytes() << " bytes, d[0] = " << u32(d[0])  << std::endl)

	this->PushMessage(Ptr<Message>(
			new NetworkDataFromClientReceivedMessage(this, c, d)
		));
}



//override
void ClientDisconnectedMessage::Handle(){
	this->c->SetState(Client::DISCONNECTED);

	//remove the client from cast it listening to, if it does
	if(this->c->isListening){
		(*this->c->listeningCast).RemoveListener(this->c);
		this->c->isListening = false;
	}

	//if client has a playing cast then notify listeners that the cast has finished
	//and remove the cast
	if(this->c->isPlaying){
		this->t->RemoveCast(this->c->playingCast);
	}
}



//override
void NetworkDataFromClientReceivedMessage::Handle(){
	ASSERT(this->data.Size() > 0)
	switch(this->data[0]){
		case 0x00: //log in
			this->thr->HandleLoginFromClient(this->client, this->data);
			break;
		case 0x01: //listen next player
			this->thr->HandleListenNextCommandFromClient(this->client);
			break;
		case 0x02:
			this->thr->HandleStopListeningCommandFromClient(this->client);
			break;
		case 0x03: //playing
			this->thr->HandlePlayingDataFromClient(this->client, this->data);
			break;
		case 0x04: //register request
			this->thr->HandleRegisterFromClient(this->client, this->data);
			break;
		default:
			ASSERT_INFO(false, "unknown packet ID = " << u32(this->data[0]))
			break;
	}
}



void Therver::HandleRegisterFromClient(ting::Ref< ::Client>& c, ting::Array<ting::byte> d){
	ASSERT(c)
	ASSERT(d.Size() > 0)
	ASSERT(d[0] == 0x04) //4 = register command

	ting::uint i = 1;

	if(i + 2 > d.Size()){
		ASSERT_INFO(false, "bad formed packet")
		return;
	}
	ting::uint unLen = ting::FromNetworkFormat16(&d[i]);
	i += 2;

	if(unLen == 0){
		//empty username is not allowed
		c->SendRegisterResult(Client::REGISTERRES_SOME_ERROR);
		return;
	}

	ASSERT(unLen != 0)
	if(i + unLen > d.Size()){
		ASSERT_INFO(false, "bad formed packet")
		return;
	}
	std::string un(reinterpret_cast<char*>(&d[i]), unLen);
	i += unLen;

	if(i + 2 > d.Size()){
		ASSERT_INFO(false, "bad formed packet")
		return;
	}
	ting::uint pwLen = ting::FromNetworkFormat16(&d[i]);
	i += 2;

	if(pwLen == 0){
		//empty password is not allowed
		c->SendRegisterResult(Client::REGISTERRES_SOME_ERROR);
		return;
	}

	ASSERT(pwLen != 0)
	if(i + pwLen > d.Size()){
		ASSERT_INFO(false, "bad formed packet")
		return;
	}
	std::string pw(reinterpret_cast<char*>(&d[i]), pwLen);

	ASSERT(i + pwLen == d.Size())

	//send request to DatabaseThread
	this->dbThread.PushMessage(Ptr<Message>(
			new RegisterNewUserMessage(
					&this->dbThread,
					c,
					un,
					pw
				)
		));
}



void Therver::HandleLoginFromClient(ting::Ref< ::Client>& c, ting::Array<ting::byte> p){
	ASSERT(c)
	ASSERT(p.Size() > 0)
	ASSERT(p[0] == 0)// 0 = login command

	//vars to be filled
	std::string un;
	std::string pw;

	ting::uint i = 1;

	//parse username
	{
		if(i + 2 > p.Size()){
			TRACE(<< "Therver::HandleLoginFromClient(): bad formed login packet !!! IGNORING !!!" << std::endl)
			return;
		}
//		TRACE(<< "unLen = " << u32(d[i]) << " " << u32(d[i + 1]) << std::endl)
		ting::uint usernameLength = ting::FromNetworkFormat16(&p[i]);
		i += 2;

		if(usernameLength == 0){
			TRACE(<< "Therver::HandleLoginFromClient(): empty login is not allowed" << std::endl)
			c->SendLoginResult(false);
			return;
		}

		ASSERT(usernameLength != 0)
		if(i + usernameLength > p.Size()){
			TRACE(<< "Therver::HandleLoginFromClient(): bad formed login packet !!! IGNORING !!!" << std::endl)
			return;
		}

		ASSERT(i < p.Size())
		un = std::string(reinterpret_cast<char*>(&p[i]), usernameLength);
		i += usernameLength;
	}

	//parse password
	{
		if(i + 2 > p.Size()){
			TRACE(<< "Therver::HandleLoginFromClient(): bad formed login packet !!! IGNORING !!!" << std::endl)
			return;
		}
//		TRACE(<< "pwLen = " << u32(d[i]) << " " << u32(d[i + 1]) << std::endl)
		ting::uint passwordLength = ting::FromNetworkFormat16(&p[i]);
		i += 2;

		if(passwordLength == 0){
			TRACE(<< "Therver::HandleLoginFromClient(): empty password is not allowed" << std::endl)
			c->SendLoginResult(false);
			return;
		}

		ASSERT(passwordLength != 0)
		if(i + passwordLength > p.Size()){
			TRACE(<< "Therver::HandleLoginFromClient(): bad formed login packet !!! IGNORING !!!" << std::endl)
			return;
		}

		ASSERT(i < p.Size())
		pw = std::string(reinterpret_cast<char*>(&p[i]), passwordLength);
		i += passwordLength;
	}

	ASSERT(i == p.Size())

	c->username = un;
	c->password = pw;

	c->SetState(Client::LOGGING_IN);

	//send query request to DB thread
	this->dbThread.PushMessage(Ptr<Message>(
			new QueryUserPasswordMessage(
					&this->dbThread,
					c,
					c->username
				)
		));
}



void Therver::HandlePlayingDataFromClient(ting::Ref< ::Client>& c, ting::Array<ting::byte> d){
	//if client is not logged in then ignore that
	if(c->IsNotLoggedIn()){
		ASSERT_INFO(false, "client is not logged in")
		return;
	}

	//if the client does not play and does not have its own cast
	//then create one, because we received playing data from player
	//which means it started playing
	if(!c->isPlaying){
		//create new cast
		this->casts.push_back(Cast(c));

		ASSERT(this->casts.size() > 0)
		T_CastsIter castIter = (--this->casts.end());

		c->isPlaying = true;
		c->playingCast = castIter;
	}

	ASSERT(c->isPlaying)

	c->playingCast->PushNewMusicDataPacket(d);
}



void Therver::HandleStopListeningCommandFromClient(ting::Ref<Client>& c){
	if(!c->isListening){
		//already not listening
		return;
	}

	ASSERT(c->listeningCast != this->casts.end())

	c->isListening = false;
	(*c->listeningCast).RemoveListener(c);
}



void Therver::HandleListenNextCommandFromClient(ting::Ref< ::Client>& c){
	//if client is not logged in then ignore that
	if(c->IsNotLoggedIn()){
		ASSERT_INFO(false, "client is not logged in")
		return;
	}

	if(!c->isListening){
		c->listeningCast = this->casts.begin();
		c->isListening = true;
	}else{
		ASSERT(c->listeningCast != this->casts.end())
		(*c->listeningCast).RemoveListener(c);
		++c->listeningCast;
	}

	//if we reached end try to jump to the beginning
	if(c->listeningCast == this->casts.end()){
		if(this->casts.size() > 0){
			c->listeningCast = this->casts.begin();
			ASSERT(c->listeningCast != this->casts.end())
		}else{
			//there is no one to listen to
			c->isListening = false;
			c->listeningCast = this->casts.end();
			c->SendStopListening();//send 'stop listening' to client
			return;
		}
	}

	(*c->listeningCast).AddListenerAndSendDataToClient(c);
}



T_CastsIter Therver::RemoveCast(T_CastsIter castIter){
	ASSERT(castIter != this->casts.end())
	Cast& cast = *castIter;

	T_CastsIter nextCastIter = castIter;
	++nextCastIter;

	if(nextCastIter == this->casts.end()){
		if(this->casts.size() > 1){
			nextCastIter = this->casts.begin();
		}
	}

	ASSERT(nextCastIter != castIter)

	if(nextCastIter != this->casts.end()){
		Cast& nextCast = *nextCastIter;

		while(Ref<Client> c = cast.PopListener()){
			ASSERT(c->isListening)
			ASSERT(c->listeningCast == castIter)

			c->listeningCast = nextCastIter;
			nextCast.AddListenerAndSendDataToClient(c);
		}//~while
	}else{//there is no one to listen to
		while(Ref<Client> c = cast.PopListener()){
			ASSERT(c->isListening)
			ASSERT(c->listeningCast == castIter)

			c->isListening = false;
			c->listeningCast = this->casts.end();
			c->SendStopListening();//send 'stop listening' to client
		}//~while
	}

	if(Ref<Client> plr = cast.GetPlayer()){
		plr->isPlaying = false;
		plr->playingCast = this->casts.end();
	}

	return this->casts.erase(castIter);
}



//override
void UserPasswordQueriedMessage::Handle(){
	ASSERT(this->c)
	
	if(this->res != SUCCESS){
		this->c->SetState(Client::CONNECTED);
		this->c->SendLoginResult(false);//send login failure
		return;
	}

	//compare passwords
	if(this->c->password.compare(this->pw) != 0){
		this->c->SetState(Client::CONNECTED);
		this->c->SendLoginResult(false);//send login failure
		return;
	}

	this->c->SetState(Client::LOGGED_IN);
	this->c->SendLoginResult(true);//send login success
}



void Therver::PushRegisterResultMessage(Client::ERegisterRes result, ting::Ref<Client> c){
	this->PushMessage(Ptr<Message>(
			new NewUserRegisteredMessage(this, c, result)
		));
}



//override
void NewUserRegisteredMessage::Handle(){
	this->c->SendRegisterResult(this->res);
}



//override
u32 Therver::KillIdleCastsTimer::OnExpire(){
	this->t->PushMessage(Ptr<Message>(
			new KillIdleCastsMessage(this->t)
		));
	return 0;//do not reschedule timer
}



void Therver::KillIdleCasts(){
	TRACE(<< "Therver::KillIdleCasts(): invoked" << std::endl)
	u32 curTicks = ting::GetTicks();

	for(T_CastsIter i = this->casts.begin(); i != this->casts.end(); ){
		if(curTicks - (*i).GetLastChunkArrivalTimestamp() >= Therver::DIdleCastDuration()){
			i = this->RemoveCast(i);
			TRACE(<< "cast killed" << std::endl)
		}else{
			++i;
		}
	}

	this->killIdleCastsTimer.StartTimer();//schedule timer again
}
